/*
    Copyright 2004,2005,2006,2007,2008,2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER     "0.2"
#define BUFFSZ  4096
#define USER    "User="
#define PASS    "Pass="

#ifndef WIN32
    #define strnicmp strncasecmp
#endif
#ifndef u_char
    #define u_char  unsigned char
#endif



void flashfxp_pwddec(u_char *name, u_char *pwd);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  buff[BUFFSZ + 1],
            name[BUFFSZ + 1],
            user[BUFFSZ + 1],
            *p;

    setbuf(stdout, NULL);

    fputs("\n"
        "FlashFXP passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <file.DAT>\n"
            "\n"
            "example paths:\n"
            "  \"C:\\Documents and Settings\\All Users\\Application Data\\FlashFXP\\3\\Sites.dat\"\n"
            "  \"%%APPDATA%%\\FlashFXP\\3\\Sites.dat\"\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("- open file %s\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    name[0] = 0;
    while(fgets(buff, BUFFSZ, fd)) {
        for(p = buff; *p && (*p != '\r') && (*p != '\n'); p++);
        *p = 0;

        if(*buff == '[') {
            p = strrchr(buff, ']');
            if(p) *p = 0;
            strcpy(name, buff + 1);

        } else if(!strnicmp(buff, USER, sizeof(USER) - 1)) {
            strcpy(user, buff);

        } else if(!strnicmp(buff, PASS, sizeof(PASS) - 1) && buff[sizeof(PASS) - 1]) {
            flashfxp_pwddec(name, buff + sizeof(PASS) - 1);
            printf("\n  [%s]\n    %s\n    %s\n", name, user, buff);
        }
    }

    fclose(fd);
    fputs("\n  Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void flashfxp_pwddec(u_char *name, u_char *pwd) {
    u_char  *enc,
            *p;
    int     len,
            old,
            new;

    if(!strchr(name, 0x03)) name = "yA36zA48dEhfrvghGRg57h5UlDv3";

    enc = name;
    len = strlen(pwd) >> 1;
    if(!len) return;
    len--;
    sscanf(pwd, "%02X", &old);
    p = pwd;
    pwd += 2;
    while(len--) {
        sscanf(pwd, "%02X", &new);
        new ^= *enc++;
        if(old >= new) new--;
        *p++ = new - old;
        sscanf(pwd, "%02X", &old);
        pwd += 2;
        if(!*enc) enc = name;
    }
    *p = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n  Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


